#' Validate the data and create a Rmd string of a component containing a 3D scatter plot.
#'
#' @param x Numeric observations mapped to the x-axis (numeric vector or data.frame).
#' @param y Numeric observations mapped to the y-axis (numeric vector or data.frame).
#' @param z Numeric observations mapped to the z-axis (numeric vector or data.frame).
#' @param x_title The title of the x-axis.
#' @param y_title The title of the y-axis.
#' @param z_title The title of the z-axis.
#' @param colour The title of the y-axis.
#' @param title The title of the component
#'
#' @return An Rmarkdown sting for the rendered component.
#' @export
scatter3D <- function(i2dashboard, x, y, z, colour = NULL, x_title = NULL, y_title = NULL, z_title = NULL, title = NULL){

  library(assertive.types)
  library(dplyr)

  # validate required parameters

  # convert vectors to data.frames:
  if(is.numeric(x)) x <- data.frame(X = x)
  if(is.numeric(y)) y <- data.frame(Y = y)
  if(is.numeric(z)) z <- data.frame(Z = z)
  if(is.character(colour) | is.numeric(colour) | is.factor(colour)) colour <- data.frame("colour" <- colour)

  assertive.types::assert_is_data.frame(x)
  assertive.types::assert_is_data.frame(y)
  assertive.types::assert_is_data.frame(z)

  # select columns only containing numeric or integer values
  x %<>%
    as.data.frame() %>%
    dplyr::select_if(function(col) is.integer(col) | is.numeric(col))
  y %<>%
    as.data.frame() %>%
    dplyr::select_if(function(col) is.integer(col) | is.numeric(col))
  z %<>%
    as.data.frame() %>%
    dplyr::select_if(function(col) is.integer(col) | is.numeric(col))

  # provide column names
  if(is.null(colnames(x))) colnames(x) <- paste0("X_", 1:ncol(x))
  if(is.null(colnames(y))) colnames(y) <- paste0("Y_", 1:ncol(y))
  if(is.null(colnames(z))) colnames(z) <- paste0("Z_", 1:ncol(z))

  # check correct dimensions
  if(nrow(x) != nrow(y)) stop("The number of rows in 'x' and 'y' is not equal.")
  if(nrow(x) != nrow(z)) stop("The number of rows in 'x' and 'z' is not equal.")

  # validate optional parameters
  if(!is.null(colour)){
    assertive.types::assert_is_data.frame(colour)
    colour %<>%
      as.data.frame() %>%
      dplyr::select_if(function(col) is.integer(col) | is.numeric(col) | is.factor(col))
    if(is.null(colnames(colour))) colnames(colour) <- paste0("Color_", 1:ncol(colour))
    if(nrow(x) != nrow(colour)) stop("The number of rows in 'x' and 'colour' is not equal.")
  }
  # check titles
  if(!is.null(title)) assertive.types::is_character(title)
  if(!is.null(x_title)) assertive.types::is_character(x_title)
  if(!is.null(y_title)) assertive.types::is_character(y_title)
  if(!is.null(z_title)) assertive.types::is_character(z_title)

  # Create random env id
  env_id <- paste0("env_", stringi::stri_rand_strings(1, 6, pattern = "[A-Za-z0-9]"))

  # Create component environment and save as .rds file
  env <- new.env()
  env$x <- x
  env$y <- y
  env$z <- z
  env$colour <- colour
  env$x_title <- x_title
  env$y_title <- y_title
  env$z_title <- z_title

  saveRDS(env, file = file.path(i2dashboard@datadir, paste0(env_id, ".rds")))

  # Generate the Rmarkdown string of the component based on the template
  timestamp <- Sys.time()
  expanded_component <- knitr::knit_expand(
    file = system.file("templates", "scatter3D.Rmd", package = "MyExtension"),
    title = title,
    env_id = env_id,
    date = timestamp)
  return(expanded_component)
}
